
module Dsc

  module Auto

    module HttpClient

        def parsed_data_file(_filename)
          filename = _filename.end_with?('.json') ? _filename : "#{_filename}.json"
          @logger.debug("File : #{@data_files_path}/#{filename}")
          parse_json(File.read("#{@data_files_path}/#{filename}"))
        end

        def get_header
          {
              :content_type => @content_type,
              :accept => '*/*',
              :cookies => @cookie
          }
        end

        def get_request_header
          if @access_token
            {
                :content_type => @content_type,
                :accept => '*/*',
                :authorization => "Bearer #{@access_token}",
                :cookies => @cookie,
                'User-Agent' => @user_agent
            }
          else
            get_header
          end
        end

        # Takes a hash of params and builds a query string for api calls
        # == Parameters:
        # params::
        # hash of params - { key_one: "ValueOne", key_two: "ValueTwo" }
        # => 'key_one=ValueOne?key_two=ValueTwo'
        def build_query(_params = {})
          res = ''
          _params.each do |key, val|
            if val.kind_of?(Array)
              val.each do |v|
                res << "#{key}=#{v}&"
              end
            else
              res << "#{key}=#{val}&"
            end
          end

          res
        end

        def query_string_to_hash(_query_string)
          Hash[CGI.parse(_query_string).map { |key, values| [key.to_sym, values[0] ||= true] }]
        end

        def response_has_body?(response_object)
          response_object.respond_to?('body')
        end

        def response_has_code?(response_object)
          response_object.respond_to?('code')
        end


        def http_request(_method, _path, _args = {})

          verify_ssl  = _args[:verify_ssl] == false ? false : true
          response = begin

            request = {
                url: _path,
                method: _method,
                verify_ssl: verify_ssl,
            }
            payload = _args[:data] || _args[:payload]
            request[:payload] = payload
            request[:headers] = _args[:headers] || { :content_type => :json, :accept => '*/*' }

            if _args[:user]
              request[:user] = _args[:user]
            end

            if _args[:password]
              request[:password] = _args[:password]
            end

            @logger.debug("RestClient Request : \r\n#{request}")

            RestClient::Request.execute(request)

          rescue RestClient::ExceptionWithResponse => e
            unless _args[:surpress_call_error]
              @logger.error("Dsc::Auto::HttpClient ERROR \r\n \
                            This may be expected if using this client to test an API\r\n\s \
                                    Base URI : #{@base_uri}\r\n\s \
                                    API VERSION : /api/v#{@version} \r\n\s \
                                    METHOD : #{_method.upcase}\r\n\s \
                                    PATH : #{_path} \r\n\s \
                                    PARAMETERS : #{_args}\r\n\s \
                                    HEADERS : #{request[:headers]}\r\n"
              )
              @logger.error(e.message)
              @logger.error(e.backtrace[0..7].join("\n"))
              error_code = e.response.code if response_has_code?(e.response)
              error_body = e.response.body if response_has_body?(e.response)
              @logger.error("ERROR API RESPONSE HEADERS - #{e.response.headers}") if e.response.headers
              @logger.error("ERROR API RESPONSE CODE - #{error_code}") if error_code
              @logger.error("ERROR API RESPONSE BODY - #{error_body}") if error_body
            end

            e.response

          end

          @history << response

          response
        end ## http_request

        # A symbol declaring the http method to use must be :get, :post, :put, :delete
        ### calls http method on base_uri
        def call(_method, _path, _args = {})

          query = nil
          error_code = nil
          error_body = nil

          path = "#{@base_uri}#{_path}"
          headers = _args[:headers] || get_request_header

          payload = _args[:payload] || _args[:data]

          ### if get and pass in data, create url params out of the data hash
          if _method == :get && payload && payload.respond_to?(:each) && payload.length > 0
            query = build_query( payload )
            path << "?#{query}"
          end

          @logger.debug("DSC::Auto::RestClient.Call : #{query ? "GET, #{path}" :
                            "#{_method.upcase}, #{path}, #{_args}"}, #{headers}\r\n")

          ### default args for get , delete - no payload should be just URL and headers
          http_request_args = [ _method, path, { headers: headers }]
          ### add data to args if post or update
          if _method == :post || :put
            payload = _args[:payload] || _args[:data]
            if payload
              http_request_args[2][:payload] = payload.to_json
            end

          end

          if _args[:user]
            http_request_args[2][:user] = _args[:user]
          end

          if _args[:password]
            http_request_args[2][:password] = _args[:password]
          end

          http_request_args[2][:surpress_call_error] = _args[:surpress_call_error]

          response = self.send(:http_request, *http_request_args)

          @response = response
        end


        ### HTTP Methods to be called on @base_uri
        ### _path will be appended to @base_url

        def get(_path, _args = {})
          call(:get, _path, _args)
        end

        def post(_path, _args = {})
          call(:post, _path, _args)
        end

        def put(_path, _args = {})
          call(:put, _path, _args)
        end

        def delete(_path, _args = {})
          call(:delete, _path, _args)
        end

        ### HTTP methods to be called on external hosts

        def get_external(_path, _args = {})
          http_request(:get, _path, _args)
        end

        def post_external(_path, _args = {})
          http_request(:post, _path, _args)
        end

        def put_external(_path, _args = {})
          http_request(:put, _path, _args)
        end

        def delete_external(_path, _args = {})
          http_request(:delete, _path, _args)
        end



    end

  end

end
